@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -30,6 +30,13 @@ abstract class CControllerPopupItemTest extends CController {
 	const ZBX_TEST_TYPE_ITEM_PROTOTYPE = 1;
 	const ZBX_TEST_TYPE_LLD = 2;
 
+	/**
+	 * Max-length of input fields that can contain resolved macro values. Used in views for input fields.
+	 *
+	 * @var int
+	 */
+	public const INPUT_MAX_LENGTH = 2048;
+
 	/**
 	 * Define a set of item types allowed to test and item properties needed to collect for each item type.
 	 *
@@ -37,7 +44,7 @@ abstract class CControllerPopupItemTest extends CController {
 	 */
 	private static $testable_item_types = [ITEM_TYPE_ZABBIX, ITEM_TYPE_SIMPLE, ITEM_TYPE_INTERNAL, ITEM_TYPE_AGGREGATE,
 		ITEM_TYPE_EXTERNAL, ITEM_TYPE_DB_MONITOR, ITEM_TYPE_HTTPAGENT, ITEM_TYPE_SSH, ITEM_TYPE_TELNET, ITEM_TYPE_JMX,
-		ITEM_TYPE_CALCULATED
+		ITEM_TYPE_CALCULATED, ITEM_TYPE_SNMP, ITEM_TYPE_SCRIPT
 	];
 
 	/**
@@ -50,7 +57,32 @@ abstract class CControllerPopupItemTest extends CController {
 	 *
 	 * @var array
 	 */
-	protected $items_require_interface = [ITEM_TYPE_ZABBIX, ITEM_TYPE_IPMI, ITEM_TYPE_SIMPLE, ITEM_TYPE_SNMP];
+	protected $items_require_interface = [
+		ITEM_TYPE_ZABBIX => [
+			'address' => true,
+			'port' => true
+		],
+		ITEM_TYPE_IPMI => [
+			'address' => true,
+			'port' => true
+		],
+		ITEM_TYPE_SIMPLE => [
+			'address' => true,
+			'port' => false
+		],
+		ITEM_TYPE_SNMP => [
+			'address' => true,
+			'port' => true
+		],
+		ITEM_TYPE_SSH => [
+			'address' => true,
+			'port' => false
+		],
+		ITEM_TYPE_TELNET => [
+			'address' => true,
+			'port' => false
+		]
+	];
 
 	/**
 	 * Item types with proxy support.
@@ -59,7 +91,7 @@ abstract class CControllerPopupItemTest extends CController {
 	 */
 	protected $items_support_proxy = [ITEM_TYPE_ZABBIX, ITEM_TYPE_SIMPLE, ITEM_TYPE_INTERNAL, ITEM_TYPE_EXTERNAL,
 		ITEM_TYPE_DB_MONITOR, ITEM_TYPE_HTTPAGENT, ITEM_TYPE_IPMI, ITEM_TYPE_SSH, ITEM_TYPE_TELNET, ITEM_TYPE_JMX,
-		ITEM_TYPE_SNMP
+		ITEM_TYPE_SNMP, ITEM_TYPE_SCRIPT
 	];
 
 	/**
@@ -105,53 +137,65 @@ abstract class CControllerPopupItemTest extends CController {
 		'url' => [
 			'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
 			'interface' => ['{HOST.IP}', '{IPADDRESS}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
-			'item' => ['{ITEM.ID}', '{ITEM.KEY}'],
+			'item' => ['{ITEM.ID}', '{ITEM.KEY.ORIG}', '{ITEM.KEY}'],
 			'support_user_macros' => true,
 			'support_lld_macros' => true
 		],
 		'posts' => [
 			'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
 			'interface' => ['{HOST.IP}', '{IPADDRESS}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
-			'item' => ['{ITEM.ID}', '{ITEM.KEY}'],
+			'item' => ['{ITEM.ID}', '{ITEM.KEY.ORIG}', '{ITEM.KEY}'],
 			'support_user_macros' => true,
 			'support_lld_macros' => true
 		],
 		'http_proxy' => [
 			'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
 			'interface' => ['{HOST.IP}', '{IPADDRESS}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
-			'item' => ['{ITEM.ID}', '{ITEM.KEY}'],
+			'item' => ['{ITEM.ID}', '{ITEM.KEY.ORIG}', '{ITEM.KEY}'],
 			'support_user_macros' => true,
 			'support_lld_macros' => true
 		],
 		'ssl_cert_file' => [
 			'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
 			'interface' => ['{HOST.IP}', '{IPADDRESS}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
-			'item' => ['{ITEM.ID}', '{ITEM.KEY}'],
+			'item' => ['{ITEM.ID}', '{ITEM.KEY.ORIG}', '{ITEM.KEY}'],
 			'support_user_macros' => true,
 			'support_lld_macros' => true
 		],
 		'ssl_key_file' => [
 			'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
 			'interface' => ['{HOST.IP}', '{IPADDRESS}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
-			'item' => ['{ITEM.ID}', '{ITEM.KEY}'],
+			'item' => ['{ITEM.ID}', '{ITEM.KEY.ORIG}', '{ITEM.KEY}'],
 			'support_user_macros' => true,
 			'support_lld_macros' => true
 		],
 		'query_fields' => [
 			'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
 			'interface' => ['{HOST.IP}', '{IPADDRESS}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
-			'item' => ['{ITEM.ID}', '{ITEM.KEY}'],
+			'item' => ['{ITEM.ID}', '{ITEM.KEY.ORIG}', '{ITEM.KEY}'],
 			'support_user_macros' => true,
 			'support_lld_macros' => true
 		],
 		'headers' => [
 			'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
 			'interface' => ['{HOST.IP}', '{IPADDRESS}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
-			'item' => ['{ITEM.ID}', '{ITEM.KEY}'],
+			'item' => ['{ITEM.ID}', '{ITEM.KEY.ORIG}', '{ITEM.KEY}'],
 			'support_user_macros' => true,
 			'support_lld_macros' => true
 		],
-		'params_f' => [
+		'parameters' => [
+			'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
+			'interface' => ['{HOST.IP}', '{IPADDRESS}', '{HOST.DNS}', '{HOST.CONN}'],
+			'item' => ['{ITEM.ID}', '{ITEM.KEY.ORIG}', '{ITEM.KEY}'],
+			'support_user_macros' => true,
+			'support_lld_macros' => true
+		],
+		'params_f' => [],
+		'script' => [
+			'support_user_macros' => true,
+			'support_lld_macros' => true
+		],
+		'timeout' => [
 			'support_user_macros' => true,
 			'support_lld_macros' => true
 		],
@@ -162,6 +206,22 @@ abstract class CControllerPopupItemTest extends CController {
 		'snmp_oid' => [
 			'support_user_macros' => true,
 			'support_lld_macros' => true
+		],
+		'username' => [
+			'support_user_macros' => true,
+			'support_lld_macros' => true
+		],
+		'password' => [
+			'support_user_macros' => true,
+			'support_lld_macros' => true
+		],
+		'http_username' => [
+			'support_user_macros' => true,
+			'support_lld_macros' => true
+		],
+		'http_password' => [
+			'support_user_macros' => true,
+			'support_lld_macros' => true
 		]
 	];
 
@@ -206,14 +266,13 @@ abstract class CControllerPopupItemTest extends CController {
 	/**
 	 * Get testable item types based on host type.
 	 *
-	 * @param int $hostid
+	 * @param string $hostid
 	 *
 	 * @return array
 	 */
-	public static function getTestableItemTypes(int $hostid = 0): array {
+	public static function getTestableItemTypes(string $hostid = '0'): array {
 		if ($hostid != 0 && self::isItemTypeTestable($hostid)) {
 			self::$testable_item_types[] = ITEM_TYPE_IPMI;
-			self::$testable_item_types[] = ITEM_TYPE_SNMP;
 		}
 
 		return self::$testable_item_types;
@@ -222,11 +281,11 @@ abstract class CControllerPopupItemTest extends CController {
 	/**
 	 * Function checks if item type can be tested depending on what type of host it belongs to.
 	 *
-	 * @param int $hostid
+	 * @param string $hostid
 	 *
 	 * @return bool
 	 */
-	protected static function isItemTypeTestable(int $hostid): bool {
+	protected static function isItemTypeTestable(string $hostid): bool {
 		$ret = (bool) API::Template()->get([
 			'countOutput' => true,
 			'templateids' => [$hostid]
@@ -280,7 +339,7 @@ abstract class CControllerPopupItemTest extends CController {
 	 *
 	 * @return CItem|CItemPrototype|CDiscoveryRule
 	 */
-	protected function getPreprocessingItemClassInstance($test_type) {
+	protected static function getPreprocessingItemClassInstance($test_type) {
 		switch ($test_type) {
 			case self::ZBX_TEST_TYPE_ITEM:
 				return new CItem;
@@ -367,6 +426,14 @@ abstract class CControllerPopupItemTest extends CController {
 			$interface_input['address'] = $input['address'];
 		}
 
+		if (array_key_exists('data', $input) && array_key_exists('interface_details', $input['data'])
+				&& is_array($input['data']['interface_details'])) {
+			$interface_input['details'] = $input['data']['interface_details'];
+		}
+		elseif (array_key_exists('interface', $input) && array_key_exists('details', $input['interface'])) {
+			$interface_input['details'] = $input['interface']['details'];
+		}
+
 		// Set proxy.
 		if (in_array($this->item_type, $this->items_support_proxy)) {
 			if (array_key_exists('data', $input) && array_key_exists('proxy_hostid', $input['data'])) {
@@ -559,7 +626,8 @@ abstract class CControllerPopupItemTest extends CController {
 					'authtype' => array_key_exists('authtype', $input) ? $input['authtype'] : ITEM_AUTHTYPE_PASSWORD,
 					'params_es' => array_key_exists('params_es', $input) ? $input['params_es'] : ITEM_AUTHTYPE_PASSWORD,
 					'username' => array_key_exists('username', $input) ? $input['username'] : null,
-					'password' => array_key_exists('password', $input) ? $input['password'] : null
+					'password' => array_key_exists('password', $input) ? $input['password'] : null,
+					'interface' => $this->getHostInterface($interface_input)
 				];
 
 				if ($data['authtype'] == ITEM_AUTHTYPE_PUBLICKEY) {
@@ -575,7 +643,8 @@ abstract class CControllerPopupItemTest extends CController {
 					'key' => $input['key'],
 					'params_es' => array_key_exists('params_es', $input) ? $input['params_es'] : null,
 					'username' => array_key_exists('username', $input) ? $input['username'] : null,
-					'password' => array_key_exists('password', $input) ? $input['password'] : null
+					'password' => array_key_exists('password', $input) ? $input['password'] : null,
+					'interface' => $this->getHostInterface($interface_input)
 				];
 				break;
 
@@ -610,6 +679,15 @@ abstract class CControllerPopupItemTest extends CController {
 					$data['interface']['dns'],  $data['interface']['port']
 				);
 				break;
+
+			case ITEM_TYPE_SCRIPT:
+				$data += [
+					'key' => $input['key'],
+					'parameters' => array_key_exists('parameters', $input) ? $input['parameters'] : [],
+					'script' => array_key_exists('script', $input) ? $input['script'] : null,
+					'timeout' => array_key_exists('timeout', $input) ? $input['timeout'] : null
+				];
+				break;
 		}
 
 		return $data;
@@ -627,42 +705,96 @@ abstract class CControllerPopupItemTest extends CController {
 			'address' => '',
 			'port' => '',
 			'interfaceid' => 0,
+			'type' => INTERFACE_TYPE_UNKNOWN,
 			'ip' => '',
 			'dns' => '',
-			'useip' => INTERFACE_USE_DNS
+			'useip' => INTERFACE_USE_DNS,
+			'details' => [
+				'community' => '',
+				'version' => SNMP_V2C,
+				'securityname' => '',
+				'securitylevel' => ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV,
+				'authpassphrase' => '',
+				'privpassphrase' => '',
+				'authprotocol' => ITEM_AUTHPROTOCOL_MD5,
+				'privprotocol' => ITEM_PRIVPROTOCOL_DES,
+				'contextname' => ''
+			]
 		];
 
+		if ($this->item_type != ITEM_TYPE_SNMP) {
+			unset($interface_data['details'], $inputs['details']);
+		}
+
 		// Get values from database; resolve macros.
 		if (($this->host['status'] == HOST_STATUS_MONITORED || $this->host['status'] == HOST_STATUS_NOT_MONITORED)
 				&& array_key_exists('interfaceid', $inputs)) {
-			$interfaces = array_key_exists('interfaceid', $inputs)
-				? API::HostInterface()->get([
-					'output' => ['hostid', 'type', 'dns', 'ip', 'port', 'main', 'useip', 'details'],
+			$output = ['hostid', 'type', 'dns', 'ip', 'port', 'main', 'useip'];
+			$interfaces = [];
+
+			if ($this->item_type == ITEM_TYPE_SNMP) {
+				$output[] = 'details';
+			}
+
+			if (itemTypeInterface($this->item_type) === false) {
+				$host_interfaces = API::HostInterface()->get([
+					'output' => $output,
+					'hostids' => $this->host['hostid'],
+					'filter' => ['main' => INTERFACE_PRIMARY]
+				]);
+				$host_interfaces = zbx_toHash($host_interfaces, 'type');
+
+				$ordered_interface_types = [INTERFACE_TYPE_AGENT, INTERFACE_TYPE_SNMP, INTERFACE_TYPE_JMX,
+					INTERFACE_TYPE_IPMI
+				];
+
+				foreach ($ordered_interface_types as $interface_type) {
+					if (array_key_exists($interface_type, $host_interfaces)) {
+						$interfaces[] = $host_interfaces[$interface_type];
+						break;
+					}
+				}
+			}
+			else {
+				$interfaces = API::HostInterface()->get([
+					'output' => $output,
 					'interfaceids' => $inputs['interfaceid'],
 					'hostids' => $this->host['hostid']
-				])
-				: [];
+				]);
+			}
 
-			if (count($interfaces) > 0) {
+			if (count($interfaces) != 0) {
 				$interfaces = CMacrosResolverHelper::resolveHostInterfaces($interfaces);
+				$interface_data = ($this->item_type == ITEM_TYPE_SNMP)
+					? ['details' => $interfaces[0]['details'] + $interface_data['details']]
+					: [];
 
-				$interface_data = [
+				$interface_data += [
 					'address' => ($interfaces[0]['useip'] == INTERFACE_USE_IP)
 						? $interfaces[0]['ip']
 						: $interfaces[0]['dns'],
 					'port' => $interfaces[0]['port'],
 					'useip' => $interfaces[0]['useip'],
+					'type' => $interfaces[0]['type'],
 					'ip' => $interfaces[0]['ip'],
 					'dns' => $interfaces[0]['dns'],
-					'interfaceid' => $interfaces[0]['interfaceid'],
-					'details' => $interfaces[0]['details']
+					'interfaceid' => $interfaces[0]['interfaceid']
 				];
 			}
 		}
 
+		if ($this->item_type == ITEM_TYPE_SCRIPT) {
+			return $interface_data;
+		}
+
 		// Apply client side cache.
 		foreach ($inputs as $key => $value) {
-			$interface_data[$key] = $value;
+			if (is_array($value)) {
+				$interface_data[$key] = $value + $interface_data[$key];
+			}
+			else {
+				$interface_data[$key] = $value;
+			}
 		}
 
 		return $interface_data;
@@ -709,14 +841,46 @@ abstract class CControllerPopupItemTest extends CController {
 					unset($data[$key]);
 				}
 			}
-			elseif ($key === 'interface') {
-				continue;
+			elseif ($key === 'interface' && $this->item_type == ITEM_TYPE_SNMP) {
+				if ($data['interface']['details']['version'] == SNMP_V3) {
+					unset($data['interface']['details']['community']);
+
+					if ($data['interface']['details']['securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV) {
+						unset($data['interface']['details']['authprotocol'],
+							$data['interface']['details']['authpassphrase'],
+							$data['interface']['details']['privprotocol'],
+							$data['interface']['details']['privpassphrase']
+						);
+					}
+					elseif ($data['interface']['details']['securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV) {
+						unset($data['interface']['details']['privprotocol'],
+							$data['interface']['details']['privpassphrase']
+						);
+					}
+				}
+				else {
+					unset($data['interface']['details']['contextname'],
+						$data['interface']['details']['securityname'],
+						$data['interface']['details']['securitylevel'],
+						$data['interface']['details']['authprotocol'],
+						$data['interface']['details']['authpassphrase'],
+						$data['interface']['details']['privprotocol'],
+						$data['interface']['details']['privpassphrase']
+					);
+				}
+
+				unset($data['interface']['type']);
 			}
 			elseif ($key === 'query_fields') {
 				if ($value === '[]') {
 					unset($data[$key]);
 				}
 			}
+			elseif ($key === 'parameters') {
+				if (!$value) {
+					unset($data[$key]);
+				}
+			}
 			elseif ($value === '' || $value === null) {
 				unset($data[$key]);
 			}
@@ -750,10 +914,31 @@ abstract class CControllerPopupItemTest extends CController {
 				'{ITEM.ID}' => (array_key_exists('itemid', $inputs) && $inputs['itemid'])
 					? $inputs['itemid']
 					: UNRESOLVED_MACRO_STRING,
-				'{ITEM.KEY}' => array_key_exists('key', $inputs) ? $inputs['key'] : UNRESOLVED_MACRO_STRING
+				'{ITEM.KEY}' => array_key_exists('key', $inputs) ? $inputs['key'] : UNRESOLVED_MACRO_STRING,
+				'{ITEM.KEY.ORIG}' => array_key_exists('key', $inputs) ? $inputs['key'] : UNRESOLVED_MACRO_STRING
 			]
 		];
 
+		if (array_key_exists('key', $inputs) && strstr($inputs['key'], '{') !== false) {
+			$usermacros = CMacrosResolverHelper::extractItemTestMacros([
+				'steps' => [],
+				'delay' => '',
+				'supported_macros' => array_diff_key($this->macros_by_item_props['key'],
+					['support_user_macros' => true, 'support_lld_macros' => true]
+				),
+				'support_lldmacros' => ($this->preproc_item instanceof CItemPrototype),
+				'texts_support_macros' => [$inputs['key']],
+				'texts_support_lld_macros' => [$inputs['key']],
+				'texts_support_user_macros' => [$inputs['key']],
+				'hostid' => $this->host ? $this->host['hostid'] : 0,
+				'macros_values' => array_intersect_key($macros, $this->macros_by_item_props['key'])
+			]);
+
+			foreach ($usermacros['macros'] as $macro => $value) {
+				$macros['item']['{ITEM.KEY}'] = str_replace($macro, $value, $macros['item']['{ITEM.KEY}']);
+			}
+		}
+
 		return $macros;
 	}
 
@@ -780,6 +965,30 @@ abstract class CControllerPopupItemTest extends CController {
 		return json_encode($result);
 	}
 
+	/**
+	 * Transform front-end familiar array of parameters fields to the form server is capable to handle. Server expects
+	 * one object where parameter names are keys and parameter values are values. Note that parameter names are unique.
+	 *
+	 * @param array $data
+	 * @param array $data[name]   Indexed array of names.
+	 * @param array $data[value]  Indexed array of values.
+	 *
+	 * @return array
+	 */
+	protected function transformParametersFields(array $data): array {
+		$result = [];
+
+		if (array_key_exists('name', $data) && array_key_exists('value', $data)) {
+			foreach (array_keys($data['name']) as $num) {
+				if (array_key_exists($num, $data['value']) && $data['name'][$num] !== '') {
+					$result += [$data['name'][$num] => $data['value'][$num]];
+				}
+			}
+		}
+
+		return $result;
+	}
+
 	/**
 	 * Transform front-end familiar array of http header fields to the form server is capable to handle.
 	 *
@@ -842,6 +1051,67 @@ abstract class CControllerPopupItemTest extends CController {
 		return $steps;
 	}
 
+	/**
+	 * Resolve macros used in the calculates item formula.
+	 *
+	 * @param string $formula  Calculated item formula.
+	 *
+	 * @return array
+	 */
+	private function resolveCalcFormulaMacros(string $formula) {
+		$macros_posted = $this->getInput('macros', []);
+
+		if (!$macros_posted) {
+			return $formula;
+		}
+
+		$expression_data = new CTriggerExpression([
+			'calculated' => true,
+			'lldmacros' => ($this->preproc_item instanceof CItemPrototype)
+		]);
+
+		if (($result = $expression_data->parse($formula)) === false) {
+			// Cannot parse a calculated item formula. Return as is.
+			return $formula;
+		}
+
+		$expression = [];
+		$pos_left = 0;
+
+		foreach ($result->getTokens() as $token) {
+			switch ($token['type']) {
+				case CTriggerExprParserResult::TOKEN_TYPE_USER_MACRO:
+				case CTriggerExprParserResult::TOKEN_TYPE_LLD_MACRO:
+				case CTriggerExprParserResult::TOKEN_TYPE_STRING:
+					if ($pos_left != $token['pos']) {
+						$expression[] = substr($formula, $pos_left, $token['pos'] - $pos_left);
+					}
+					$pos_left = $token['pos'] + $token['length'];
+					break;
+			}
+
+			switch ($token['type']) {
+				case CTriggerExprParserResult::TOKEN_TYPE_USER_MACRO:
+				case CTriggerExprParserResult::TOKEN_TYPE_LLD_MACRO:
+					$expression[] = array_key_exists($token['value'], $macros_posted)
+						? CTriggerExpression::quoteString($macros_posted[$token['value']], false)
+						: $token['value'];
+					break;
+
+				case CTriggerExprParserResult::TOKEN_TYPE_STRING:
+					$string = strtr($token['data']['string'], $macros_posted);
+					$expression[] = CTriggerExpression::quoteString($string, false, true);
+					break;
+			}
+		}
+
+		if ($pos_left != strlen($formula)) {
+			$expression[] = substr($formula, $pos_left);
+		}
+
+		return implode('', $expression);
+	}
+
 	/**
 	 * Resolve macros used in item property fields.
 	 *
@@ -858,6 +1128,12 @@ abstract class CControllerPopupItemTest extends CController {
 				continue;
 			}
 
+			// Special processing for calculated item formula.
+			if ($field === 'params_f') {
+				$inputs[$field] = $this->resolveCalcFormulaMacros($inputs[$field], $macros_posted);
+				continue;
+			}
+
 			// Construct array of supported macros.
 			$types = [
 				'usermacros' => true,
@@ -875,7 +1151,7 @@ abstract class CControllerPopupItemTest extends CController {
 			}
 
 			// Get strings to resolve and types of supported macros.
-			if ($field === 'query_fields' || $field === 'headers') {
+			if ($field === 'query_fields' || $field === 'headers' || $field === 'parameters') {
 				if (!array_key_exists($field, $inputs) || !$inputs[$field]) {
 					continue;
 				}
@@ -912,6 +1188,24 @@ abstract class CControllerPopupItemTest extends CController {
 			}
 		}
 
+		// Resolve interface details (SNMP) macros separately.
+		if (array_key_exists('interface', $inputs) && array_key_exists('details', $inputs['interface'])) {
+			foreach ($inputs['interface']['details'] as &$field) {
+				if (strstr($field, '{') !== false) {
+					$matched_macros = (new CMacrosResolverGeneral)->getMacroPositions($field, ['usermacros' => true]);
+
+					foreach (array_reverse($matched_macros, true) as $pos => $macro) {
+						$macro_value = array_key_exists($macro, $macros_posted)
+							? $macros_posted[$macro]
+							: '';
+
+						$field = substr_replace($field, $macro_value, $pos, strlen($macro));
+					}
+				}
+			}
+			unset($field);
+		}
+
 		return $inputs;
 	}
 
@@ -932,4 +1226,45 @@ abstract class CControllerPopupItemTest extends CController {
 
 		return $value;
 	}
+
+
+	/**
+	 * Validates interface object in context of current item type.
+	 *
+	 * @param array  $interface
+	 * @param string $interface['address']               (optional)
+	 * @param string $interface['port']                  (optional)
+	 * @param array  $interface['details']               (optional)
+	 * @param int    $interface['details']['version']
+	 * @param string $interface['details']['community']  (optional)
+	 *
+	 * @return bool
+	 */
+	final protected function validateInterface(array $interface): bool {
+		if ($this->item_type == ITEM_TYPE_SNMP) {
+			if (($interface['details']['version'] == SNMP_V1 || $interface['details']['version'] == SNMP_V2C)
+					&& (!array_key_exists('community', $interface['details'])
+						|| $interface['details']['community'] === '')) {
+				error(_s('Incorrect value for field "%1$s": %2$s.', _('SNMP community'), _('cannot be empty')));
+
+				return false;
+			}
+		}
+
+		if ($this->items_require_interface[$this->item_type]['address']
+				&& (!array_key_exists('address', $interface) || $interface['address'] === '')) {
+			error(_s('Incorrect value for field "%1$s": %2$s.', _('Host address'), _('cannot be empty')));
+
+			return false;
+		}
+
+		if ($this->items_require_interface[$this->item_type]['port']
+				&& (!array_key_exists('port', $interface) || $interface['port'] === '')) {
+			error(_s('Incorrect value for field "%1$s": %2$s.', _('Port'), _('cannot be empty')));
+
+			return false;
+		}
+
+		return true;
+	}
 }
