@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 if ($data['graph']['unavailable_object']) {
 	$item = (new CTableInfo())->setNoDataMessage(_('No permissions to referred object or it does not exist!'));
 
@@ -28,17 +32,16 @@ if ($data['graph']['unavailable_object']) {
 	];
 }
 else {
-	$flickerfree_item = (new CDiv())
-		->addItem((new CLink(null, $data['item_graph_url']))
+	$flickerfree_item = (new CDiv(
+		(($data['item_graph_url'] !== null) ? new CLink(null, $data['item_graph_url']) : new CSpan(null))
 			->setId($data['graph']['containerid'])
 			->addClass(ZBX_STYLE_DASHBRD_WIDGET_GRAPH_LINK)
-		)
+	))
 		->addClass('flickerfreescreen')
-		->setAttribute('data-timestamp', $data['graph']['timestamp'])
 		->setId('flickerfreescreen_'.$data['graph']['dataid']);
 
-	$script = 'timeControl.addObject("'.$data['graph']['dataid'].'", '.CJs::encodeJson($data['timeline']).', '.
-			CJs::encodeJson($data['time_control_data']).
+	$script = 'timeControl.addObject("'.$data['graph']['dataid'].'", '.json_encode($data['timeline']).', '.
+			json_encode($data['time_control_data']).
 		');'.
 		'timeControl.processObjects();'.
 		'window.flickerfreeScreen.add('.zbx_jsvalue($data['fs_data']).');';
@@ -88,6 +91,16 @@ else {
 				'}'.
 			'}'.
 
+			'if (typeof zbx_graph_widget_edit_start !== typeof(Function)) {'.
+				'function zbx_graph_widget_edit_start(img_id) {'.
+					'var $link = jQuery("#" + img_id).parent();'.
+
+					'$link'.
+						'.attr("href", "javascript:void(0)")'.
+						'.attr("role", "button");'.
+				'}'.
+			'}'.
+
 			'jQuery(".dashbrd-grid-container").dashboardGrid("addAction", "onResizeEnd", '.
 				'"zbx_graph_widget_resize_end", "'.$data['widget']['uniqueid'].'", {'.
 					'parameters: ["'.$data['graph']['dataid'].'"],'.
@@ -104,6 +117,12 @@ else {
 				'"zbx_graph_widget_delete", "'.$data['widget']['uniqueid'].'", {'.
 					'parameters: ["'.$data['graph']['dataid'].'",'.zbx_jsvalue($data['fs_data']).'],'.
 					'trigger_name: "graph_widget_delete_'.$data['widget']['uniqueid'].'"'.
+				'});'.
+
+			'jQuery(".dashbrd-grid-container").dashboardGrid("addAction", "onEditStart", '.
+				'"zbx_graph_widget_edit_start", "'.$data['widget']['uniqueid'].'", {'.
+					'parameters: ["'.$data['graph']['dataid'].'"],'.
+					'trigger_name: "graph_widget_edit_start_'.$data['widget']['uniqueid'].'"'.
 				'});';
 	}
 
@@ -123,4 +142,4 @@ if ($data['user']['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
 	$output['debug'] = CProfiler::getInstance()->make()->toString();
 }
 
-echo (new CJson())->encode($output);
+echo json_encode($output);
