@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,6 +24,10 @@
  */
 class CProblem extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER]
+	];
+
 	protected $tableName = 'problem';
 	protected $tableAlias = 'p';
 	protected $sortColumns = ['eventid'];
@@ -71,7 +75,6 @@ class CProblem extends CApiService {
 			'any'						=> null,	// (internal) true if need not filtred by r_eventid
 			'evaltype'					=> TAG_EVAL_TYPE_AND_OR,
 			'tags'						=> null,
-			'recent'					=> null,
 			'filter'					=> null,
 			'search'					=> null,
 			'searchByAny'				=> null,
@@ -288,8 +291,7 @@ class CProblem extends CApiService {
 
 		// recent
 		if ($options['recent'] !== null && $options['recent']) {
-			$config = select_config();
-			$ok_events_from = time() - timeUnitToSeconds($config['ok_period']);
+			$ok_events_from = time() - timeUnitToSeconds(CSettingsHelper::get(CSettingsHelper::OK_PERIOD));
 
 			$sqlParts['where'][] = '(p.r_eventid IS NULL OR p.r_clock>'.$ok_events_from.')';
 		}
@@ -334,7 +336,7 @@ class CProblem extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($event = DBfetch($res)) {
 			if ($options['countOutput']) {
 				$result = $event['rowscount'];
@@ -401,6 +403,23 @@ class CProblem extends CApiService {
 
 		$eventids = array_keys($result);
 
+		// Adding operational data.
+		if ($this->outputIsRequested('opdata', $options['output'])) {
+			$problems = DBFetchArrayAssoc(DBselect(
+				'SELECT p.eventid,p.clock,p.ns,t.triggerid,t.expression,t.opdata'.
+				' FROM problem p'.
+				' JOIN triggers t ON t.triggerid=p.objectid'.
+				' WHERE '.dbConditionInt('p.eventid', $eventids)
+			), 'eventid');
+
+			foreach ($result as $eventid => $problem) {
+				$result[$eventid]['opdata'] =
+					(array_key_exists($eventid, $problems) && $problems[$eventid]['opdata'] !== '')
+						? CMacrosResolverHelper::resolveTriggerOpdata($problems[$eventid], ['events' => true])
+						: '';
+			}
+		}
+
 		// adding acknowledges
 		if ($options['selectAcknowledges'] !== null) {
 			if ($options['selectAcknowledges'] != API_OUTPUT_COUNT) {
@@ -486,6 +505,43 @@ class CProblem extends CApiService {
 			unset($row);
 		}
 
+		// Resolve webhook urls.
+		if ($this->outputIsRequested('urls', $options['output'])) {
+			$tags_options = [
+				'output' => ['eventid', 'tag', 'value'],
+				'filter' => ['eventid' => $eventids]
+			];
+			$tags = DBselect(DB::makeSql('problem_tag', $tags_options));
+
+			$events = [];
+
+			foreach ($result as $event) {
+				$events[$event['eventid']]['tags'] = [];
+			}
+
+			while ($tag = DBfetch($tags)) {
+				$events[$tag['eventid']]['tags'][] = [
+					'tag' => $tag['tag'],
+					'value' => $tag['value']
+				];
+			}
+
+			$urls = DB::select('media_type', [
+				'output' => ['event_menu_url', 'event_menu_name'],
+				'filter' => [
+					'type' => MEDIA_TYPE_WEBHOOK,
+					'status' => MEDIA_TYPE_STATUS_ACTIVE,
+					'show_event_menu' => ZBX_EVENT_MENU_SHOW
+				]
+			]);
+
+			$events = CMacrosResolverHelper::resolveMediaTypeUrls($events, $urls);
+
+			foreach ($events as $eventid => $event) {
+				$result[$eventid]['urls'] = $event['urls'];
+			}
+		}
+
 		// Adding event tags.
 		if ($options['selectTags'] !== null && $options['selectTags'] != API_OUTPUT_COUNT) {
 			if ($options['selectTags'] === API_OUTPUT_EXTEND) {
