@@ -1,5 +1,37 @@
+<?php
+/*
+** Zabbix
+** Copyright (C) 2001-2021 Zabbix SIA
+**
+** This program is free software; you can redistribute it and/or modify
+** it under the terms of the GNU General Public License as published by
+** the Free Software Foundation; either version 2 of the License, or
+** (at your option) any later version.
+**
+** This program is distributed in the hope that it will be useful,
+** but WITHOUT ANY WARRANTY; without even the implied warranty of
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+** GNU General Public License for more details.
+**
+** You should have received a copy of the GNU General Public License
+** along with this program; if not, write to the Free Software
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+**/
+
+
+/**
+ * @var CView $this
+ */
+?>
+
 <script type="text/javascript">
 	jQuery(document).ready(function($) {
+		$('#description')
+			.on('input keydown paste', function() {
+				$('#event_name').attr('placeholder', $(this).val());
+			})
+			.trigger('input');
+
 		// Refresh field visibility on document load.
 		changeRecoveryMode();
 		changeCorrelationMode();
@@ -17,8 +49,8 @@
 
 			$('#expression_row').find('label').text(
 				(recovery_mode == <?= ZBX_RECOVERY_MODE_RECOVERY_EXPRESSION ?>)
-					? <?= CJs::encodeJson(_('Problem expression')) ?>
-					: <?= CJs::encodeJson(_('Expression')) ?>
+					? <?= json_encode(_('Problem expression')) ?>
+					: <?= json_encode(_('Expression')) ?>
 			);
 			$('.recovery_expression_constructor_row')
 				.toggle(recovery_mode == <?= ZBX_RECOVERY_MODE_RECOVERY_EXPRESSION ?>);
@@ -40,6 +72,23 @@
 					&& correlation_mode == <?= ZBX_TRIGGER_CORRELATION_TAG ?>
 				);
 		}
+
+		let triggers_initialized = false;
+		$('#tabs').on('tabscreate tabsactivate', function(event, ui) {
+			const panel = (event.type === 'tabscreate') ? ui.panel : ui.newPanel;
+
+			if (panel.attr('id') === 'triggersTab') {
+				if (triggers_initialized) {
+					return;
+				}
+
+				$('#triggersTab')
+					.find('.<?= ZBX_STYLE_TEXTAREA_FLEXIBLE ?>')
+					.textareaFlexible();
+
+				triggers_initialized = true;
+			}
+		});
 	});
 
 	/**
@@ -56,7 +105,7 @@
 			}
 
 			// return to the same form after it has been submitted
-			jQuery('#action').val(<?php echo CJs::encodeJson(getRequest('action')) ?>);
+			jQuery('#action').val(<?= json_encode(getRequest('action')) ?>);
 
 			create_var('triggersForm', 'add_dependency', 1, true);
 		}
